/**
 * jQuery jqcascading Plugin
 * Narong Rammanee
 * ranarong@gmail.com
 *
 * Latest Release: April 2012
 * 
 * 
 * ********************************** Options ***********************************
 *      - url 
 *          A string containing the URL to which the request is sent.
 *          Default: "json.php?val="
 *      
 *      - parent
 *          Parent drop down list.
 *          Default: null
 * 
 *      - child
 *          Child drop down list.
 *          Default: null
 * 
 *      - initialText
 *          Set initial text in option.
 *          Deafault: "-- Please Select --"
 *      
 *      - selectedOption
 *          Set selected option of dropdown list.
 *          Default: "-1"
 * 
 *      - waitMsg
 *          The message and indicator display before server sent respons.
 *          Default: "Loading..."
 * 
 *      - loadingImg
 *          Set loading image.
 *          Default: "images/loading4.gif"
 * 
 *      - disabled
 *          True -> set disable dropdownlist while dropdown list value is -1.
 *          Default: true
 * 
 * 
 * ******************************** How to Use **********************************
 * Parte 1 - Setup
 *      jquery cascading plugin uses the jquery JavaScript library, only. So, include just these two javascript files in your header.
 *          
 *          <script type="text/javascript" src="js/jquery.min.js"></script>
 *          <script type="text/javascript" src="js/jquery.cascading.min.js"></script>
 *      
 *      Include the CSS file responsible to style the jquery cascading plugin.
 *      
 *     <link rel="stylesheet" href="css/style.css" type="text/css">
 * Part 2 - Activate
 *      See some examples:
 *          $(function(){
 *              $("#model").cascading( { parent:"#brand", child: ["#model_des", "#model_det", "#model_dett"] } );
 *              $("#model_des").cascading( { parent:"#model", child: ["#model_det", "#model_dett"] } );
 *              $("#model_det").cascading( { parent:"#model_des", child: "#model_dett" } );
 *              $("#model_dett").cascading( { parent:"#model_det" } );
 *          });
 * 
 */

(function ($) {
    // Cascading methods.
    var cascading = {
        init : function (options) {
            return this.each(function () {
                var self, settings;
                self = $(this);                                 // Self dropdown list.
                
                settings = $.extend({
                    url            : "json.php?val=",     // Set URL.
                    parent         : null,                      // Parent dropdown list.
                    child          : null,                      // Set child.
                    initialText    : "-- Please Select --",     // Set initial text in option.
                    selectedOption : "-1",                      // Set selected option.
                    waitMsg        : "Loading...",              // Set wait message.
                    loadingImg     : "images/loading4.gif",     // Set loading image.
                    disabled       : true                       // Set disabled dropdown list.
                }, options);
                $(settings.parent).bind('change', function () {
                    // Call method chage when dropdown list is changed.
                    cascading.change(self, settings);
                });
            });
        },
        change : function (self, settings) {
            var parent, url, defaultOption, selectedOption, child, waitMsg, 
                disabled, loadingImg, parentValue, wait, loading;
            parent = $(settings.parent);
            if (parent) {
                url            = settings.url;
                selectedOption = settings.selectedOption;
                child          = settings.child;
                waitMsg        = settings.waitMsg;
                initialText    = settings.initialText
                disabled       = settings.disabled;
                loadingImg     = settings.loadingImg;
                parentValue    = parent.val();              // Get parent value.
                wait           = $("<em />");               // Create em element contain loading.
                loading        = $("<img />").attr({ src: loadingImg, alt: waitMsg });   //Set loading image.
                newOption      = $("<option></option>");    // Create new option
                defaultOption  = newOption.val("-1").text(initialText);   // Set default option.
                
                
                $.ajax({
                    type: "GET",
                    dataType: "json",
                    url: url + parentValue + '&self=' + self.attr('id'),
                    beforeSend: function () {
                        // Show loading image & message
                        wait.remove()
                            .html(loading)
                            .append(waitMsg)
                            .appendTo("h1");
                    },
                    success: function (json) {
                        // Set default option & disabled dropdown list.
                        self.html(defaultOption.clone())
                            .removeAttr("disabled", "disabled");
                            
                        if (json) {
                            // Loop json object.
                            $.each(json, function (value, text) {
                                // Append option to dropdown list.
                                self.append(newOption.clone().val(value).text(text));
                            });
                        }
                        
                        // Remove waiting image & message 
                        wait.empty();
                        
                        // Set selected option
                        if (selectedOption != "-1") {
                            self.val(selectedOption);
                        }
                        
                        // Disabled self dropdown list.
                        if (disabled && parentValue == "-1") {
                            self.attr("disabled", "disabled");
                        }
                        
                        // Reset chain
                        cascading.resetChain(defaultOption, child, disabled);
                    }
                });
            }
        },
        resetChain : function (defaultOption, child, disabled) {
            // If child is null.
            if (!child) {
                return;
            }
            
            // If child is object.
            if (typeof child === "object") {
                // Loop disabled child dropdown list.
                $.each(child, function (index, value) {
                    $(value).html(defaultOption.clone());
                    if (disabled) {
                        $(value).attr("disabled", "disabled");
                    }
                });
            } else {
                $(child).html(defaultOption.clone());
                if (disabled) {
                    $(child).attr("disabled", "disabled");
                }
            }
        }
    };
    
    $.fn.cascading = function () {
        return cascading.init.apply(this, arguments);
    };

}(jQuery));
